

set -ex



test -f $PREFIX/include/mkl.h
test -f $PREFIX/include/mkl_blas.h
test -f $PREFIX/include/mkl_cblas.h
test -f $PREFIX/include/mkl_df.h
test -f $PREFIX/include/mkl_lapack.h
test -f $PREFIX/include/mkl_service.h
test -f $PREFIX/include/mkl_solvers_ee.h
test -f $PREFIX/include/mkl_trans.h
test -f $PREFIX/include/mkl_types.h
test -f $PREFIX/include/mkl_version.h
exit 0
